* Logit Chapter
drop _all
clear


capture program drop bipoissll
program bipoissll
	version 15
	args todo b lnf
	quietly {
		local y1 "$ML_y1"
		local y2 "$ML_y2"
		tempvar  xb1 xb2 
		tempname theta eta 
		mleval `xb1'   = `b', eq(1)
		mleval `xb2'   = `b', eq(2)
		mleval `theta' = `b', eq(3) scalar

		// Bound theta for exp() function
		scalar `theta' = max(min(`theta',18),-18)
		scalar `eta'   = 1 - exp(-`theta')

		// SECTION: Calculate marginal probabilities 
		// (marginals are Poisson)
		tempvar f10 f11 f20 f21
		generate double `f10' = poisson(exp(`xb1'), `y1') 
		generate double `f11' = cond(`y1'==0, 0, ///
			poisson(exp(`xb1'), `y1'-1))
		generate double `f20' = poisson(exp(`xb2'), `y2') 
		generate double `f21' = cond(`y2'==0, 0, ///
			poisson(exp(`xb2'), `y2'-1))

		// Calculate the Frank Copula function
		tempvar ttt t1 t2 t3 t4
		generate double `t1' = - (1/`theta') * ///
			log((`eta' - (1-exp(-`theta'*`f10')) * ///
			(1-exp(-`theta'*`f20')))/`eta')
		generate double `t2' = - (1/`theta') * ///
			log((`eta' - (1-exp(-`theta'*`f11')) * ///
			(1-exp(-`theta'*`f20')))/`eta')
		generate double `t3' = - (1/`theta') * ///
			log((`eta' - (1-exp(-`theta'*`f10')) * ///
			(1-exp(-`theta'*`f21')))/`eta')
		generate double `t4' = - (1/`theta') * ///
			log((`eta' - (1-exp(-`theta'*`f11')) * ///
			(1-exp(-`theta'*`f21')))/`eta')

		// Set up for reasonable bounds of probabilities
		tempname maxv minv
		scalar `minv' = 1e-6
		scalar `maxv' = 1.0-`minv'

		// Impose Frechet-Hoeffding bounds
		replace `t1' = min(max(`t1', max( ///
			`f10'+`f20'-1,`minv')),   ///
			min(`f10', `f20',`maxv'))
		replace `t2' = min(max(`t2', max( ///
			`f11'+`f20'-1,`minv')),   ///
			min(`f11', `f20',`maxv'))
		replace `t3' = min(max(`t3', max( ///
			`f10'+`f21'-1,`minv')),   ///
			min(`f10', `f21',`maxv'))
		replace `t4' = min(max(`t4', max( ///
			`f11'+`f21'-1,`minv')),   ///
			min(`f11', `f21',`maxv'))

		// Use difference calculation to get joint pmf
		generate double `ttt' = `t1'-`t2'-`t3'+`t4'

		// Calculate log likelihood
		replace `lnf' = log(`ttt')
	}
end
drop _all
set seed 29348
set obs 1000
gen byte x11 = rbinomial(1,.5)
gen byte x12 = rbinomial(1,.5)
gen byte x21 = rbinomial(1,.5)
gen byte x22 = rbinomial(1,.5)
gen double mu1 = exp(1-x11+x12)
gen double mu2 = exp(1+x21-x22)
rejectsample y1 y2, mu1(mu1) mu2(mu2) lambda(12) marg1(margpoisson) marg2(margpoisson) jointpmf(jointfcoppp)
* Run models to get starting values for each outcome
poisson y1 x11 x12, nolog
matrix b1 = e(b)
poisson y2 x21 x22, nolog
matrix b2 = e(b)

* Define (near zero = independent) starting value for theta
matrix tt = (0.1)

* Paste together the starting value vector
matrix bb = b1, b2, tt

* Use ml to fit a bivariate model where 
* each marginal distribution is Poisson
ml model lf0 bipoissll (y1:y1 = x11 x12) ///
	(y2:y2 = x21 x22) (theta:)
ml init bb, copy
* Turn search off because we have starting values
ml maximize, search(off) nolog ///
	title(Bivariate Poisson-Poisson regression)

capture program drop binbinomll
program binbinomll
	version 15
	args todo b lnf
	quietly {
		local y1 "$ML_y1"
		local y2 "$ML_y2"
		tempvar  xb1 xb2
		tempname theta eta lnalpha1 lnalpha2
		mleval `xb1'      = `b', eq(1)
		mleval `lnalpha1' = `b', eq(2) scalar
		mleval `xb2'      = `b', eq(3)
		mleval `lnalpha2' = `b', eq(4) scalar
		mleval `theta'    = `b', eq(5) scalar

		// Bound theta for exp() function
		scalar `theta' = max(min(`theta',18),-18)
		scalar `eta'   = 1 - exp(-`theta')

		tempname m1 p1 m2 p2
		scalar `m1'     = exp(-`lnalpha1')
		generate double `p1' = 1/(1+exp(`xb1'+`lnalpha1'))

		scalar `m2'     = exp(-`lnalpha2')
		generate double `p2' = 1/(1+exp(`xb2'+`lnalpha2'))

		// Calculate marginal probabilities 
		tempvar f10 f11 f20 f21
		generate double `f10' = nbinomial(`m1', `y1', `p1')
		generate double `f11' = cond(`y1'==0, 0, ///
			nbinomial(`m1', `y1'-1, `p1'))
		generate double `f20' = nbinomial(`m2', `y2', `p2')
		generate double `f21' = cond(`y2'==0, 0, ///
			nbinomial(`m2', `y2'-1, `p2'))

		// Calculate the Frank Copula function
		tempvar ttt t1 t2 t3 t4
		generate double `t1' = - (1/`theta') * log((`eta' - ///
			(1-exp(-`theta'*`f10')) * ///
			(1-exp(-`theta'*`f20')))/`eta')
		generate double `t2' = - (1/`theta') * log((`eta' - /// 
			(1-exp(-`theta'*`f11')) * /// 
			(1-exp(-`theta'*`f20')))/`eta')
		generate double `t3' = - (1/`theta') * log((`eta' - /// 
			(1-exp(-`theta'*`f10')) * /// 
			(1-exp(-`theta'*`f21')))/`eta')
		generate double `t4' = - (1/`theta') * log((`eta' - /// 
			(1-exp(-`theta'*`f11')) * /// 
			(1-exp(-`theta'*`f21')))/`eta')

		// Set up for reasonable bounds of probabilities
		tempname maxv minv
		scalar `minv' = 1e-6
		scalar `maxv' = 1.0-`minv'

		// Impose Frechet-Hoeffding bounds
		replace `t1' = min(max(`t1', max( ///
			`f10'+`f20'-1,`minv')),   ///
			min(`f10', `f20',`maxv'))
		replace `t2' = min(max(`t2', max( ///
			`f11'+`f20'-1,`minv')),   ///
			min(`f11', `f20',`maxv'))
		replace `t3' = min(max(`t3', max( ///
			`f10'+`f21'-1,`minv')),   ///
			min(`f10', `f21',`maxv'))
		replace `t4' = min(max(`t4', max( ///
			`f11'+`f21'-1,`minv')),   ///
			min(`f11', `f21',`maxv'))

		// Use difference calculation to get joint pmf
		generate double `ttt' = `t1'-`t2'-`t3'+`t4'

		// Calculate log likelihood
		replace `lnf' = log(`ttt')
	}
end

drop _all
set seed 89238
set obs 1000
gen byte x11 = rbinomial(1,.5)
gen byte x12 = rbinomial(1,.5)
gen byte x21 = rbinomial(1,.5)
gen byte x22 = rbinomial(1,.5)
gen double mu1 = exp(1-x11+x12)
gen double mu2 = exp(1+x21-x22)
rejectsample y1 y2, mu1(mu1) mu2(mu2) lambda(2) ap1(0.5) ap2(4.0) marg1(margnbinom) marg2(margnbinom) jointpmf(jointfcopnn)

* Use ml to fit a bivariate model where 
* each marginal distribution is negative binomial
ml model lf0 binbinomll (y1:y1 = x11 x12) (lnalpha1:) ///
	(y2:y2 = x21 x22) (lnalpha2:) (theta:)
* Turn search on instead of using starting values
ml maximize, search(on) ltol(1e-2) tol(1e-2) nolog ///
	title(Bivariate Nbinom-Nbinom regression)

capture program drop famoyepbll
program famoyepbll
	version 15
	args todo b lnf
	quietly {
		local y1 "$ML_y1"
		local y2 "$ML_y2"
		tempname lambda
		tempvar  xb1 xb2 
		mleval `xb1'     = `b', eq(1)
		mleval `xb2'     = `b', eq(2)
		mleval `lambda'  = `b', eq(3) scalar

		tempvar mu1 mu2 
		generate double `mu1' = exp(`xb1')
		generate double `mu2' = exp(`xb2')

		tempname dd
		scalar `dd'      = 1 - exp(-1)

		replace `lnf' = `y1'*`xb1'+`y2'*`xb2'-`mu1'-`mu2'- ///
			lngamma(`y1'+1) - lngamma(`y2'+1) + ///
			log(1+`lambda'*(exp(-`y1')-exp(-`dd'*`mu1')) * ///
			(exp(-`y2')-exp(-`dd'*`mu2')))
	}
end

drop _all
set seed 74238
set obs 1000
gen byte x11 = rbinomial(1,.5)
gen byte x12 = rbinomial(1,.5)
gen byte x21 = rbinomial(1,.5)
gen byte x22 = rbinomial(1,.5)
gen double mu1 = exp(1-x11+x12)
gen double mu2 = exp(1+x21-x22)
rejectsample y1 y2, mu1(mu1) mu2(mu2) lambda(20) marg1(margpoisson) marg2(margpoisson) jointpmf(jointfamp)
poisson y1 x11 x12, nolog
matrix a1 = e(b)
poisson y2 x21 x22, nolog
matrix a2 = e(b)
matrix tt = (0.0)
matrix aa = a1, a2, tt
ml model lf0 famoyepbll (y1:y1 = x11 x12) (y2:y2 = x21 x22) (lambda:)
ml init aa, copy
ml maximize, search(off) title(Famoye Biv Poisson regression) nolog

capture program drop mobinbll
program mobinbll
	version 15
	args todo b lnf
	quietly {
		local y1 "$ML_y1"
		local y2 "$ML_y2"
		tempvar  xb1 xb2
		tempname lnalpha
		mleval `xb1'      = `b', eq(1)
		mleval `xb2'      = `b', eq(2)
		mleval `lnalpha'  = `b', eq(3) scalar

		tempvar mu1 mu2 alpha den
		generate double `mu1'   = exp(`xb1'+`lnalpha')
		generate double `mu2'   = exp(`xb2'+`lnalpha')
		generate double `alpha' = exp(-`lnalpha')
		generate double `den'   = `mu1'+`mu2'+1

		tempvar p1 p2 p3
		generate double `p1' = `mu1'/`den'
		generate double `p2' = `mu2'/`den'
		generate double `p3' = 1/`den'

		replace `lnf' = `y1'*log(`p1') + ///
			`y2'*log(`p2') + `alpha'*log(`p3') + ///
			lngamma(`y1'+`y2'+`alpha') - ///
			lngamma(`y1'+1) - lngamma(`y2'+1) - ///
			lngamma(`alpha')
	}
end

drop _all
set seed 82222
set obs 1000
gen byte x11 = rbinomial(1,.5)
gen byte x12 = rbinomial(1,.5)
gen byte x21 = rbinomial(1,.5)
gen byte x22 = rbinomial(1,.5)
gen double mu1 = exp(1-x11+x12)
gen double mu2 = exp(1+x21-x22)
rejectsample y1 y2, mu1(mu1) mu2(mu2) lambda(2.0) marg1(margnbinom) marg2(margnbinom) jointpmf(jointmo)
poisson y1 x11 x12, nolog
matrix a1 = e(b)
poisson y2 x21 x22, nolog
matrix a2 = e(b)
matrix tt = (0.0)
matrix aa = a1, a2, tt
ml model lf0 mobinbll (y1:y1 = x11 x12) (y2:y2 = x21 x22) (lnalpha:)
ml init aa, copy
ml maximize, search(off) title(Marshall-Olkin Biv Negbin regression) nolog

capture program drop fbinbll
program fbinbll
	version 15
	args todo b lnf
	quietly {
		local y1 "$ML_y1"
		local y2 "$ML_y2"
		tempvar  xb1 xb2
		tempname lnalpha1 lnalpha2 lambda
		mleval `xb1'      = `b', eq(1)
		mleval `lnalpha1' = `b', eq(2) scalar
		mleval `xb2'      = `b', eq(3)
		mleval `lnalpha2' = `b', eq(4) scalar
		mleval `lambda'   = `b', eq(5) scalar

		tempname alpha1 alpha2
		scalar `alpha1' = exp(-`lnalpha1')
		scalar `alpha2' = exp(-`lnalpha2')

		tempname dd m1 m2
		scalar `dd' = 1 - exp(-1)
		scalar `m1' = 1/`alpha1'
		scalar `m2' = 1/`alpha2'

		tempvar mu1 mu2 c1 c2
		generate double `mu1' = exp(`xb1')
		generate double `mu2' = exp(`xb2')
		generate double `c1'  = (1+`dd'*`mu1'*`m1')^(-1/`m1')
		generate double `c2'  = (1+`dd'*`mu2'*`m2')^(-1/`m2')

		tempname j1 j2
		summarize `y1'
		local j1 = r(max)-1
		summarize `y2'
		local j2 = r(max)-1

		replace `lnf' = log(1+`lambda'*(exp(-`y1')-`c1') * ///
			(exp(-`y2')-`c2'))
		forvalues t=1/2 {
			forvalues j=0/`j`t'' {
				replace `lnf' = `lnf' + log( ///
					1/`m`t'' + `j') if `y`t'' > `j'
			}
			replace `lnf' = `lnf' + `y`t''*log(`mu`t'') - ///
				(1/`m`t'')*log(`m`t'') - ///
				(`y`t''+1/`m`t'') * ///
				log(`mu`t''+1/`m`t'') - lngamma(`y`t''+1)
		}
	}
end

drop _all
set seed 13002
set obs 1000
gen byte x11 = rbinomial(1,.5)
gen byte x12 = rbinomial(1,.5)
gen byte x21 = rbinomial(1,.5)
gen byte x22 = rbinomial(1,.5)
gen double mu1 = exp(1-x11+x12)
gen double mu2 = exp(1+x21-x22)
rejectsample y1 y2, mu1(mu1) mu2(mu2) ap1(0.5) ap2(1.5) lambda(2.0) marg1(margnbinom) marg2(margnbinom) jointpmf(jointfamn)
nbreg y1 x11 x12, nolog
matrix b1 = e(b)
nbreg y2 x21 x22, nolog
matrix b2 = e(b)
matrix tt = (0.5)
matrix bb = b1, b2, tt
ml model lf0 fbinbll (y1:y1 = x11 x12) (lnalpha1:) (y2:y2 = x21 x22) (lnalpha2:) (lambda:)
ml init bb, copy
ml maximize, search(off) title(Famoye Biv Negbin regression) nolog

