* Gamma Chapter
drop _all

set seed 238947                     /* Set seed for reproducibility */
set obs 10000                       /* Number of obs = 10,000       */
generate x1 = rnormal()             /* Creation of random variates  */
generate x2 = rnormal()
generate x3 = rnormal()
generate d  = 50+5*int((_n-1)/1000) /* Creation of 10 different denominators */
tabulate d                          /* Tabulation of the denominator         */

set seed 214231
generate xb = 1 + .5*x1 - .75*x2 + .25*x3    /* Create linear predictor */
generate p  = exp(xb)/(1+exp(xb))
generate y  = rbinomial(d,p)                 /* Create simulated dataset */
glm y x1 x2 x3, family(binomial d) nolog     /* Model y on x1, x2, and x3 */
save sim_data, replace

capture program drop GlogitSim
program GlogitSim, rclass
    quietly {
        drop _all
        set obs 10000
        generate x1 = rnormal()
        generate x2 = rnormal()
        generate x3 = rnormal()
        generate d  = 50+5*int((_n-1)/1000)
        generate xb = 1 + 0.5*x1 - .75*x2 + .25*x3
        generate p  = exp(xb)/(1+exp(xb))
        generate y  = rbinomial(d,p)
        glm y x1 x2 x3, family(binomial d)
    }
    return scalar sx1 = _b[x1]
    return scalar sx2 = _b[x2]
    return scalar sx3 = _b[x3]
    return scalar sxc = _b[_cons]
    return scalar pds =  e(dispers_p)
end

set seed 2934288
simulate x1=r(sx1) x2=r(sx2) x3=r(sx3) cons=r(sxc) disp=r(pds), reps(100): GlogitSim
summarize

use sim_data
glm y x2 x3, family(binomial d) nolog   /* Here we omit the x1 covariate */

set seed 64321
generate x23 = x2*x3                          /* create interaction            */
generate xb1 = 1 + .5*x1 - .75*x2 + .25*x3 + .2*x23 /* create linear predictor */
generate p1  = exp(xb1)/(1+exp(xb1))          /* create probability of success */
generate y1  = rbinomial(d, p1)               /* create simulated dataset      */

glm y1 x1 x2 x3 x23, family(binomial d) nolog

glm y1 x1 x2 x3, family(binomial d) nolog


use heart01, clear
glm death anterior hcabg kk2 kk3 kk4 age2-age4, family(binomial) irls scale(x2) eform noheader nolog

egen grp=group(anterior-age4)    /* The covariate pattern is  */
                                 /* assigned a group number   */
drop if grp == .                 /* Discard missing values    */
egen cases = count(grp), by(grp) /* Size of covariate pattern */ 
egen die = total(death), by(grp) /* Deaths in pattern         */
sort grp                         /* Sort by covariate pattern */
quietly by grp: keep if _n==1    /* Keep one case per pattern */

glm die anterior hcabg kk2 kk3 kk4 age2-age4, family(binomial cases) eform nolog

glm die anterior hcabg kk2 kk3 kk4 age2-age4, family(binomial cases) scale(x2) eform noheader nolog

glm die anterior hcabg kk2 kk3 kk4 age2-age4, family(binomial cases) link(loglog) scale(x2) eform nolog

display chiprob(1, 3.14)

generate t = _n
tsset t
glm die anterior hcabg kk2 kk3 kk4 age2-age4, family(binomial cases) link(loglog) irls nwest(anderson 2) nolog

williams die cases anterior hcabg kk2 kk3 kk4 age2-age4, eform nolog
