* Quasilikelihood Chapter
drop _all

use accident
glm y i.site i.variety, family(binomial) link(logit) vce(robust) nolog 

glm y i.site i.variety, family(binomial) link(logit) scale(x2) nolog 

predict double eta, eta
predict double pearson, pearson
predict double mu
generate double logvar = log(mu*mu*(1-mu)*(1-mu))
label var pearson "Pearson residuals"
label var eta     "Linear predictor"
label var logvar  "Log(variance)"
twoway scatter pearson eta, yline(0)
twoway scatter pearson logvar, yline(0) 

twoway scatter pearson eta, yline(0)

twoway scatter pearson logvar, yline(0) 


capture drop eta
capture drop mu
capture drop pearson
capture drop logvar
                                                                                                                             
capture program drop binsq
program binsq
        args todo eta mu return touse
                                                                                                                             
        if `todo' == -1 {
                if "$SGLM_m" != "1" {                           /* Title */
                        global SGLM_vt "Quasi"
                        global SGLM_vf "u^2*(1-u/$SGLM_m)^2"
                }
                else {
                        global SGLM_vt "Quasi"
                        global SGLM_vf "u^2*(1-u)^2"
                }
                                                                                                                             
                local y     "$SGLM_y"
                local m     "$SGLM_m"
                local touse "`eta'"
                                                                                                                             
                capture assert `m'>0 if `touse'         /* sic, > */
                if _rc {
                        display as error`"`m' has nonpositive values"'
                        exit 499
                }
                capture assert `m'==int(`m') if `touse'
                if _rc {
                        display as error `"`m' has noninteger values"'
                        exit 499
                }
                capture assert `y'>=0 if `touse'
                if _rc {
                        display as error`"dependent variable `y' has negative values"'
                        exit 499
                }
                capture assert `y'<=`m' if `touse'
                if _rc {
                        noisily display as error`"`y' > `m' in some cases"'
                        exit 499
                }
                                                                                                                             
                global SGLM_mu "glim_mu 0 $SGLM_m"
                exit
        }
        if `todo' == 0 {                        /* Initialize eta */
                if "$SGLM_L" == "glim_l01" {                    /* Identity */
                        generate double `eta' = `mu'/$SGLM_m
                }
                else if "$SGLM_L" == "glim_l02" {               /* Logit */
                        generate double `eta' = ln(`mu'/($SGLM_m-`mu'))
                }
                else if "$SGLM_L" == "glim_l03" {               /* Log */
                        generate double `eta' = ln(`mu'/$SGLM_m)
                }
                else if "$SGLM_L" == "glim_l05" {               /* Log compl */
                        generate double `eta' = ln(1-`mu'/$SGLM_m)
                }
                else if "$SGLM_L" == "glim_l06" {               /* Loglog */
                        generate double `eta' = -ln(-ln(`mu'/$SGLM_m))
                }
                else if "$SGLM_L" == "glim_l07" {               /* Cloglog */
                        generate double `eta' = ln(-ln(1-`mu'/$SGLM_m))
                }
                 else if "$SGLM_L" == "glim_l08" {               /* Probit */
                        generate double `eta' = invnormal(`mu'/$SGLM_m)
                }
                else if "$SGLM_L" == "glim_l09" {               /* Reciprocal */
                        generate double `eta' = $SGLM_m/`mu'
                }
                else if "$SGLM_L" == "glim_l10" {               /* Power(-2) */
                        generate double `eta' = ($SGLM_m/`mu')^2
                }
                else if "$SGLM_L" == "glim_l11" {               /* Power(a) */
                        generate double `eta' = (`mu'/$SGLM_m)^$SGLM_a
                }
                else if "$SGLM_L" == "glim_l12" {               /* OPower(a) */
                        generate double `eta' = /*
                                */ ((`mu'/($SGLM_m-`mu'))^$SGLM_a-1) / $SGLM_a
                }
                else {
                        generate double `eta' = $SGLM_m*($SGLM_y+.5)/($SGLM_m+1)
                }
                exit
        }
        if `todo' == 1 {                        /* V(mu) */
                generate double `return' =  `mu'*`mu'*(1-`mu'/$SGLM_m)*(1-`mu'/$SGLM_m)
                exit
        }
        if `todo' == 2 {                        /* (d V)/(d mu) */
                generate double `return' =  2*`mu' - 6*`mu'*`mu'/$SGLM_m + /*
                        */ 4*`mu'*`mu'*`mu'/($SGLM_m*$SGLM_m)
                exit
        }
        if `todo' == 3 {                        /* deviance */
                noisily display as error "deviance calculation not supported"
                generate double `return' = 0
                exit
        }
        if `todo' == 4 {                        /* Anscombe */
                noisily display as error "Anscombe residuals not supported"
                exit 198
         }
        if `todo' == 5 {                        /* ln-likelihood */
                local y "$SGLM_y"
                local m "$SGLM_m"
                if "`y'" == "" {
                        local y "`e(depvar)'"
                }
                if "`m'" == "" {
                        local m "`e(m)'"
                }
                generate double `return' = (2*`y'-1)*ln(`mu'/(`m'-`mu')) - `y'/`mu' - /*
                        */ (`m'-`y')/(`m'-`mu')
                exit
        }
        if `todo' == 6 {
                noisily display as err "adjusted deviance residuals not supported"
                exit 198
        }
        noisily display as err "Unknown call to glim variance function"
        error 198
end

glm y i.site i.variety, family(binsq) link(logit) nolog 

predict double eta, eta
predict double pearson, pearson
predict double mu
generate double logvar = log(mu*mu*(1-mu)*(1-mu))
label var eta     "Linear predictor"
label var pearson "Pearson residuals"
label var logvar  "Log(variance)"
twoway scatter pearson eta, yline(0) 

twoway scatter pearson logvar, yline(0) 

capture program drop tweedie
program tweedie
	version 15
	args todo eta mu return

	if `todo' == -1 {			/* Title */
		global SGLM_vt "Tweedie"
		global SGLM_vf "phi*u^($SGLM_fa)"
		global SGLM_mu "glim_mu 0 ."
		exit
	}
	if `todo' == 0 {
		if $SGLM_p < 1e-7 {
			generate double `eta' = ln(`mu')
		}
		else {
			generate double `eta' = `mu'^$SGLM_p
		}
		exit
	}
	if `todo' == 1 {			/* V(mu) */
		generate double `return' =  `mu'^$SGLM_fa
		noisily summarize `return'
		exit 
	}
	if `todo' == 2 {			/* (d V)/(d mu) */
		generate double `return' =  ($SGLM_fa)*`mu'^($SGLM_fa-1)
		noisily summarize `return'
		exit
	}
	if `todo' == 3 {			/* deviance */
                local y "$SGLM_y"
                if (abs($SGLM_fa-2) < 1e-7) {
                        noi di as err "Tweedie variance does not support p=2"
                        exit 198
                }
                if abs($SGLM_fa-1) < 1e-7 {
                        generate double `return' = 2*(`y'*ln(`y'/`mu')-(`y'-`mu'))
                }
                else {
                        local p = $SGLM_fa
                        generate double `return' = 2*(`y'^(2-`p')/((1-`p')   ///
			        *(2-`p'))-(`y'*`mu'^(1-`p'))/(1-`p')+`mu'^(2-`p')/(2-`p'))
                }
                exit
	}
	if `todo' == 4 {			/* Anscombe */
		generate double `return' = .
		exit
	}
	if `todo' == 5 {			/* ln-likelihood */
		noisily display as err "Tweedie family supported by IRLS only"
		exit 198
	}
	if `todo' == 6 {			/* Adjustment to deviance */
		generate double `return' = 0
		exit
	}

	noisily display as err "Unknown call to glim variance function"
	error 198
end

use medpar, clear
glm los hmo white age, family(poisson) irls nolog
glm los hmo white age, family(tweedie 1) link(power 0) irls nolog

glm los hmo white age, family(gauss) irls nolog
glm los hmo white age, family(tweedie 0) link(power 1) irls nolog
